{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}file-upload-modal{% endblock %}
{% block modal_title %} Upload File {% endblock %}

{% block modal_body %}
<div>
    Choose file to upload:
    {% include 'core_main_app/user/blob/list/modals/file_upload_form.html' %}
</div>
{% endblock %}

{% block modal_footer %}
<a class="btn btn-secondary pull-left" {% if BOOTSTRAP_VERSION|first == "4" %}data-dismiss{% elif BOOTSTRAP_VERSION|first == "5"  %}data-bs-dismiss{% endif %}="modal" href="#"><i class="fas fa-times"></i> Cancel</a>
<a id="file-upload-yes" class="btn btn-secondary" href="#"><i class="fas fa-upload"></i> Upload</a>
{% endblock %}

