/** Checkbox Select Multiple CSS **/

.multiple-columns {
  columns: 120px 4;
  list-style: none;
}

.double-columns {
  columns: 270px 2;
  list-style: none;
}

.multiple-select-form{
    height :200px;
    overflow-y: scroll;
}

.multiple-select-form:hover{
    border-radius: 10px;
    border: 1px solid #d0d0d0;
}