{% load blob_tags %}

<div>
    <button id="btn-previous-page" type="button" class="btn btn-secondary mb-2 {% if BOOTSTRAP_VERSION|first == "4" %}float-right{% elif BOOTSTRAP_VERSION|first == "5"  %}float-end{% endif %}">
        <i class="fas fa-arrow-left"></i> Back to previous
    </button>
    {% if data.can_write %}
    <a class="btn btn-secondary edit
        {% if BOOTSTRAP_VERSION|first == "4" %}mr-1{% elif BOOTSTRAP_VERSION|first == "5"  %}me-1{% endif %}
        {% if BOOTSTRAP_VERSION|first == "4" %}float-right{% elif BOOTSTRAP_VERSION|first == "5"  %}float-end{% endif %}"
        href="{% url 'core_main_app_blob_metadata' data.blob.id %}">
        <i class="fas fa-list"></i>
        Manage Metadata
    </a>
    {% endif %}
    <a class="btn btn-secondary download
        {% if BOOTSTRAP_VERSION|first == "4" %}mr-1{% elif BOOTSTRAP_VERSION|first == "5"  %}me-1{% endif %}
        {% if BOOTSTRAP_VERSION|first == "4" %}float-right{% elif BOOTSTRAP_VERSION|first == "5"  %}float-end{% endif %}"
        href="{% url 'core_main_app_rest_blob_download' data.blob.id %}">
        <i class="fas fa-download"></i> Download
    </a>
</div>
