{% extends "core_main_app/_render/admin/theme/tools/box.html" %}
{%load get_attribute%}

{% block box_title %}Error{% endblock %}

{% block box_body %}
    {% include 'core_main_app/common/buttons/back_to_previous.html' %}
    <div class="alert alert-danger" style="width: 75%;">
        <i class="fas fa-exclamation-triangle"></i>
        {{ data | get_attribute:"error" }}
    </div>

{% endblock %}