# Generated by Django 3.2 on 2022-05-25 13:50

from django.db import migrations, models
import django.db.models.expressions


class Migration(migrations.Migration):

    dependencies = [
        ("core_main_app", "0003_psql_full_text"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="templateversionmanager",
            options={
                "ordering": [
                    django.db.models.expressions.OrderBy(
                        django.db.models.expressions.F("display_rank"),
                        nulls_last=True,
                    )
                ]
            },
        ),
        migrations.AddField(
            model_name="templateversionmanager",
            name="display_rank",
            field=models.IntegerField(blank=True, null=True),
        ),
    ]
