{% extends 'core_main_app/_render/admin/theme/tools/modal.html' %}

{% block modal_id %}add-metadata-modal{% endblock %}
{% block modal_title %} Add Metadata {% endblock %}

{% block modal_body %}
<div id="blob_metadata">
    Select metadata document to link to this file:
    <div id="add-metadata">
    </div>
</div>
{% endblock %}

{% block modal_footer %}
<a class="btn btn-secondary pull-left" {% if BOOTSTRAP_VERSION|first == "4" %}data-dismiss{% elif BOOTSTRAP_VERSION|first == "5"  %}data-bs-dismiss{% endif %}="modal" href="#"><i class="fas fa-times"></i> Cancel</a>
<a id="add-metadata-yes" class="btn btn-secondary" href="#"><i class="fas fa-check"></i> Add</a>
{% endblock %}

