""" Rights for core main app
"""

# Anonymous group
ANONYMOUS_GROUP = "anonymous"

# Default group
DEFAULT_GROUP = "default"

# API rights
API_CONTENT_TYPE = "api"
API_ACCESS = "api_access"

# Data
# FIXME edit and delete rights are never used
EDIT_DATA = "edit_data"
DELETE_DATA = "delete_data"
PUBLISH_DATA = "publish_data"

# Blob
PUBLISH_BLOB = "publish_blob"

# Form
EDIT_FORM = "edit_form"
DELETE_FORM = "delete_form"

# Workspace
CAN_READ_CODENAME = "can_read_workspace_"
CAN_WRITE_CODENAME = "can_write_workspace_"

CAN_READ_NAME = "Can read workspace "
CAN_WRITE_NAME = "Can write workspace "

CONTENT_TYPE_APP_LABEL = "core_main_app"
