# Generated by Django 3.2.18 on 2023-05-31 20:26

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core_main_app", "0005_template_dependencies"),
    ]

    operations = [
        migrations.AddField(
            model_name="data",
            name="_blob",
            field=models.ForeignKey(
                blank=True,
                default=None,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="_metadata",
                to="core_main_app.blob",
            ),
        ),
    ]
