# Generated by Django 4.2.18 on 2023-08-24 14:55

from django.db import migrations, models

import core_main_app.components.user_preferences.models
import core_main_app.utils.storage.storage


class Migration(migrations.Migration):

    dependencies = [
        ("core_main_app", "0006_blob_metadata"),
    ]

    operations = [
        migrations.CreateModel(
            name="UserPreferences",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("user_id", models.CharField(max_length=200, unique=True)),
                (
                    "timezone",
                    models.CharField(
                        blank=True,
                        max_length=200,
                        null=True,
                        validators=[
                            core_main_app.components.user_preferences.models.validate_timezone
                        ],
                    ),
                ),
            ],
            options={
                "verbose_name": " User Preferences",
                "verbose_name_plural": " User Preferences",
            },
        ),
    ]
