{% block box_title %}Template XSLT Rendering{% endblock %}

{% block box_tools %}

{% include 'core_main_app/common/buttons/go_to.html' with url=data.url_back_to label='Back to Versions' %}

{% endblock %}

{% block box_body %}
<div id="error-div" style="display: none;">
    <div class="alert alert-danger alert-dismissable">
        <a href="#"
           class="{% if BOOTSTRAP_VERSION|first == "4" %}close{% elif BOOTSTRAP_VERSION|first == "5"  %}btn-close{% endif %}"
           {% if BOOTSTRAP_VERSION|first == "4" %}data-dismiss{% elif BOOTSTRAP_VERSION|first == "5"  %}data-bs-dismiss{% endif %}="alert"
        aria-label="close">{% if BOOTSTRAP_VERSION|first == "4" %}&times;{% elif BOOTSTRAP_VERSION|first == "5"  %}{% endif %}</a>
        <strong>{{ data.errors }}</strong>
    </div>
</div>

<form id="form_start" method="post">
    {% csrf_token %}
    {{ data.form_template_xsl_rendering }}

    <br/>
    <button class="btn btn-secondary pull-left">
        <i class="fas fa-times"></i> Cancel
    </button>
    <button type="submit" class="btn btn-primary {% if BOOTSTRAP_VERSION|first == "4" %}float-right{% elif BOOTSTRAP_VERSION|first == "5"  %}float-end{% endif %}">
      <i class="fas fa-save"></i> Save
    </button>
</form>
{% endblock %}