# Generated by Django 3.2.18 on 2023-05-25 19:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("core_main_app", "0004_template_ordering"),
    ]

    operations = [
        migrations.AlterField(
            model_name="template",
            name="dependencies",
            field=models.ManyToManyField(
                blank=True, default=[], to="core_main_app.Template"
            ),
        ),
    ]
