<form id="add-metadata-form" class="multiple-select-form" method="post">
    {% csrf_token %}
    {{ add_metadata_form }}
    <div id="add-metadata-errors" style="color:red;">
        {{ errors | safe}}
    </div>
    <br/>
</form>