""" Migration to order Blob object by creation date.

Generated by Django 4.2.18 on 2023-08-21 13:14
"""
from django.db import migrations


class Migration(migrations.Migration):
    """Migration class."""

    dependencies = [
        ("core_main_app", "0007_user_preferences"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="blob",
            options={"ordering": ["-creation_date"]},
        ),
    ]
