.toast-container {
    z-index: 1500;
    position: fixed;
    top: 2.5rem;
    right: 1rem;

    display: flex;
    flex-direction: column;
    align-items: end;
}
.toast {
    opacity: 1;
    border-radius: 0.25rem;
    border-style: none;
    max-width: 100%;
    font-size: 1.2rem;

    flex-basis: fit-content;
    width: fit-content;
}
.toast > .toast-body {
    display: flex;
    flex-direction: row;

    align-items: center;
}
.toast > .toast-body > .toast-text {
    width: 100%;
}
.toast > .toast-body > button {
    display: flex;
    background: none;
    font-size: 0.9rem;
    opacity: 1;
}
.toast-info {
    color: #3a87ad;
    background-color: #d9edf7;
}
.toast-info > .toast-body > button {
    opacity: 1;
    color: #3a87ad;
}
.toast-success {
    color: #468846;
    background-color: #dff0d8;
}
.toast-success > .toast-body > button {
    opacity: 1;
    color: #468846;
}
.toast-warning {
    color: #c09853;
    background-color: #fcf8e3;
}
.toast-warning > .toast-body > button {
    opacity: 1;
    color: #c09853;
}
.toast-danger {
    color: #b94a48;
    background-color: #f2dede;
}
.toast-danger > .toast-body > button {
    opacity: 1;
    color: #b94a48;
}
.toast-default {
    color: #666;
    background-color: #f6f6f6;
}
.toast-default > .toast-body > button {
    opacity: 1;
    color: #666;
}